class EndangeredProductsFilter(admin.SimpleListFilter):

    title = _('Qty')
    parameter_name = 'is_endangered'

    def lookups(self, request, model_admin):
        return [('true', _('Endangered products'))]

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(min_qty__gt=models.F('qty'))
        else:
            return queryset



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-03-22 14:47
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('products', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='WishListItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('qty', models.PositiveIntegerField(default=1, verbose_name='Quantity')),
                ('product_title', models.CharField(max_length=255, verbose_name='Product title')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('product', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='products.Product', verbose_name='Product')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='wishlist_items', to=settings.AUTH_USER_MODEL, verbose_name='Owner')),
            ],
            options={
                'ordering': ['pk'],
                'verbose_name': 'Wish list item',
                'verbose_name_plural': 'Wish list items',
            },
        ),
        migrations.AlterUniqueTogether(
            name='wishlistitem',
            unique_together=set([('user', 'product')]),
        ),
    ]




from django.apps import apps
from django import template

from shop.currencies.settings import CURRENCY_SESSION_KEY

register = template.Library()


@register.assignment_tag
def get_categories():
    category_model = apps.get_model('products', 'ProductCategory')
    return category_model.objects.all()


@register.assignment_tag
def get_root_categories():
    category_model = apps.get_model('products', 'ProductCategory')
    return category_model.objects.root_nodes()


@register.simple_tag(takes_context=True)
def get_printable_product_price(context, product):

    exchange_rate_model = apps.get_model('currencies', 'ExchangeRate')

    dst_currency = context.request.session.get(CURRENCY_SESSION_KEY)

    if dst_currency is not None:
        return exchange_rate_model.convert(
            product.price.initial, product.currency, int(dst_currency),
            printable=True)

    return product.price.printable_default




from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class ProductsAppConfig(AppConfig):

    name = 'shop.products'
    verbose_name = _('Products')


default_app_config = 'shop.products.ProductsAppConfig'




from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _


class WishListItem(models.Model):

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='wishlist_items',
        verbose_name=_('Owner'))

    product = models.ForeignKey(
        'products.Product', verbose_name=_('Product'),
        on_delete=models.SET_NULL, blank=True, null=True)

    qty = models.PositiveIntegerField(_('Quantity'), default=1)

    product_title = models.CharField(_("Product title"), max_length=255)

    date_created = models.DateTimeField(
        _('Date created'), auto_now_add=True, editable=False)

    def __unicode__(self):
        return self.product.title

    class Meta:
        ordering = ['pk']
        unique_together = (('user', 'product'), )
        verbose_name = _('Wish list item')
        verbose_name_plural = _('Wish list items')



from django.conf.urls import url

from shop.wishlist import views


urlpatterns = [

    url(r'^$', views.index, name='index'),

    url(r'^add/$', views.add, name='add'),

    url(r'^remove/$', views.remove, name='remove')

]




from django.utils.translation import ugettext_lazy as _


CURRENCY_UAH = 1
CURRENCY_USD = 2
CURRENCY_EUR = 3


DEFAULT_CURRENCIES = (
    (CURRENCY_UAH, _('UAH')),
    (CURRENCY_USD, _('USD')),
    (CURRENCY_EUR, _('EUR')),
)




from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from shop.orders.constants import *


DEFAULT_DELIVERY_METHODS = (
    (DELIVERY_METHOD_SELF, _('Self delivery')),
    (DELIVERY_METHOD_NOVA_POSHTA, _('Nova Poshta')),
    (DELIVERY_METHOD_INTIME, _('In Time')),
    (DELIVERY_METHOD_GUNSEL, _('Gunsel')),
    (DELIVERY_METHOD_AVTOLUKS, _('Avtoluks')),
    (DELIVERY_METHOD_OTHER, _('Other')),
)

DEFAULT_PAYMENT_METHODS = (
    (PAYMENT_METHOD_CASH, _('Cash payment')),
    (PAYMENT_METHOD_CASHLESS, _('Cashless payment')),
    (PAYMENT_METHOD_COD, _('C.O.D.')),
    (PAYMENT_METHOD_PRIVAT24, _('Privat24 Payment')),
)

DEFAULT_STATUSES = (
    (STATUS_NOT_REVIEWED, _('Not reviewed')),
    (STATUS_PROCESSING, _('Processing')),
    (STATUS_CANCELED, _('Canceled')),
    (STATUS_COMPLETED, _('Completed')),
)

ORDER_PAYMENT_METHODS = getattr(
    settings, 'ORDER_PAYMENT_METHODS', DEFAULT_PAYMENT_METHODS)

ORDER_DELIVERY_METHODS = getattr(
    settings, 'ORDER_DELIVERY_METHODS', DEFAULT_DELIVERY_METHODS)

ORDER_STATUSES = getattr(settings, 'ORDER_STATUSES', DEFAULT_STATUSES)

ORDER_INVOICE_MOBILE = getattr(settings, 'ORDER_INVOICE_MOBILE', '')

ORDER_INVOICE_EMAIL = getattr(settings, 'ORDER_INVOICE_EMAIL', '')

DEFAULT_DELIVERY_METHOD = getattr(settings, 'DEFAULT_DELIVERY_METHOD', None)

DEFAULT_PAYMENT_METHOD = getattr(settings, 'DEFAULT_PAYMENT_METHOD', None)

DEFAULT_STATUS = getattr(settings, 'DEFAULT_STATUS', STATUS_NOT_REVIEWED)

POST_OFFICE_CHOICES = getattr(settings, 'POST_OFFICE_CHOICES', None)




from shop.products.abstract_models import (
    AbstractProduct, AbstractProductCategory, AbstractProductImage)


class ProductCategory(AbstractProductCategory):
    pass


class Product(AbstractProduct):
    pass


class ProductImage(AbstractProductImage):
    pass




from django.conf.urls import url, include
from django.views.generic import TemplateView

from shop.products import views


app_name = 'products'


urlpatterns = [

    url(r'^api/', include('shop.products.api.urls', namespace='api')),

    url(r'^$',
        TemplateView.as_view(template_name='products/categories.html'),
        name='index'),

    url(r'^search/$', views.product_search, name='search'),

    url(r'^(?P<product_slug>[\w-]*)_(?P<product_pk>\d+)/$',
        views.ProductInfoView.as_view(), name='info'),

    url(r'^category/(?P<category_slug>[\w-]+(/[\w-]+)*)_'
        r'(?P<category_pk>\d+)/$', views.product_list, name='category'),

]




from django.http import JsonResponse
from django.shortcuts import redirect
from django.views.decorators.http import require_POST

from shop.currencies.settings import CURRENCY_SESSION_KEY
from shop.currencies.forms import CurrencyForm


@require_POST
def set_currency(request):

    form = CurrencyForm(request.POST)

    if not form.is_valid():
        return JsonResponse(form.errors.as_json())

    currency = form.cleaned_data['currency']

    request.session[CURRENCY_SESSION_KEY] = currency

    return redirect(request.GET.get('next', '/'))




from django.apps import apps

from shop.currencies.lib import format_printable_price


CART_SESSION_KEY = 'CART'


class CartItem(object):

    def __init__(self, product, qty):
        self.product = product
        self.qty = int(qty)

    def serialize(self):

        product = self.product

        return {
            'product_pk': product.pk,
            'qty': self.qty,
            'price': str(product.price_in_currency),
            'currency': str(product.currency)
        }

    @property
    def default_subtotal(self):
        return self.product.price.default * self.qty

    @property
    def printable_default_subtotal(self):
        return format_printable_price(self.default_subtotal)

    @property
    def initial_subtotal(self):
        return self.product.price.initial * self.qty

    @property
    def printable_initial_subtotal(self):
        return format_printable_price(
            self.initial_subtotal, currency=self.product.currency)


class Cart(object):

    def __init__(self, session, session_key=None):
        self._items = {}
        self.session = session
        self.session_key = session_key or CART_SESSION_KEY

        if self.session_key in self.session:
            cart_representation = self.session[self.session_key]

            ids_in_cart = cart_representation.keys()

            Product = apps.get_model('products', 'Product')

            for product in Product.objects.filter(pk__in=ids_in_cart):
                item = cart_representation[str(product.pk)]
                self._items[product.pk] = CartItem(product, item['qty'])

    def update_session(self):
        self.session[self.session_key] = self.serialize()
        self.session.modified = True

    def add(self, product, qty=1):

        qty = int(qty)

        if qty < 1:
            raise ValueError('Quantity must be at least 1 when adding to cart')

        if product in self.products:
            self._items[product.pk].qty += qty
        else:
            self._items[product.pk] = CartItem(product, qty)

        self.update_session()

    def remove(self, product):
        if product in self.products:
            del self._items[product.pk]
            self.update_session()

    def clear(self):
        self._items = {}
        self.update_session()

    def set_qty(self, product, qty):
        qty = int(qty)

        if qty < 0:
            raise ValueError('qty must be positive when updating cart')

        if product in self.products:
            self._items[product.pk].qty = qty

            if self._items[product.pk].qty < 1:
                del self._items[product.pk]

            self.update_session()

    def serialize(self):

        obj = {}

        for item in self.items:
            product_id = str(item.product.pk)
            obj[product_id] = item.serialize()

        return obj

    def get_item(self, product):
        return self._items[product.pk]

    @property
    def items(self):
        return self._items.values()

    @property
    def count(self):
        return sum([item.qty for item in self.items])

    @property
    def unique_count(self):
        return len(self._items)

    @property
    def is_empty(self):
        return self.unique_count == 0

    @property
    def products(self):
        return [item.product for item in self.items]

    @property
    def default_total(self):
        return sum([item.default_subtotal for item in self.items])

    @property
    def printable_default_total(self):
        return format_printable_price(self.default_total)

    @property
    def initial_total(self):

        total = {}

        for item in self.items:

            currency = item.product.currency

            if currency in total:
                total[currency] += item.initial_subtotal
            else:
                total[currency] = item.initial_subtotal

        return total

    @property
    def printable_initial_total(self):

        totals = []

        for currency, price in self.initial_total.items():
            totals.append(format_printable_price(price, currency=currency))

        return ', '.join(totals)




from django.apps import apps
from django.shortcuts import render, get_object_or_404
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.http import require_POST
from django.http import JsonResponse

from shop.cart.lib import Cart
from shop.cart.forms import CartItemForm


def _get_product_from_request(request):

    Product = apps.get_model('products', 'Product')

    product_pk = request.POST.get('product_pk')

    return get_object_or_404(Product, pk=product_pk)


def index(request, template_name='cart/index.html'):
    return render(request, template_name)


@require_POST
def add(request):

    product = _get_product_from_request(request)

    cart = Cart(request.session)
    cart.add(product)

    return JsonResponse({
        'message': str(_('Added')),
        'total': cart.printable_default_total
    })


@require_POST
def remove(request):

    product = _get_product_from_request(request)

    cart = Cart(request.session)
    cart.remove(product)

    return JsonResponse({
        'message': str(_('%s removed from card' % product.title)),
        'total': cart.printable_default_total
    })


@require_POST
def set_qty(request):

    product = _get_product_from_request(request)

    form = CartItemForm(data=request.POST)

    if not form.is_valid():
        return JsonResponse(form.errors.as_json(), status=403)

    cart = Cart(request.session)
    cart.set_qty(product, form.cleaned_data['qty'])

    cart_item = cart.get_item(product)

    return JsonResponse({
        'message': str(_('%s quantity changed' % product.title)),
        'subtotal': cart_item.printable_default_subtotal,
        'total': cart.printable_default_total
    })




from django.apps import apps
from django.shortcuts import redirect, render, get_object_or_404
from django.contrib import messages
from django.contrib.sites.models import Site
from django.contrib.auth.decorators import login_required
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string

from easy_pdf.views import PDFTemplateView

from shop.cart.lib import Cart

from shop.orders.settings import ORDER_INVOICE_MOBILE, ORDER_INVOICE_EMAIL
from shop.orders.forms import CheckoutForm
from shop.orders.lib import send_new_order_notifications


def checkout(request, form_class=CheckoutForm):

    user = request.user

    form_params = {'data': request.POST or None}

    if user.is_authenticated():

        profile = getattr(user, 'profile', None)

        form_params['initial'] = {
            'name': user.first_name,
            'surname': user.last_name,
            'email': user.email,
            'mobile': profile.mobile if profile is not None else ''
        }

    form = form_class(**form_params)

    if request.method == 'POST' and form.is_valid():

        cart = Cart(request.session)

        order = form.save(commit=False)

        if user.is_authenticated():
            order.user = user

        order.save()

        for item in cart.items:
            product = item.product
            order.products.create(
                parent=product, title=product.title,
                price=product.price.default, qty=item.qty)

        cart.clear()

        send_new_order_notifications(order)

        message = render_to_string(
            'orders/new_order_message.html', {'order': order})

        messages.success(request, mark_safe(message))

        return redirect('home')

    return render(request, 'orders/checkout.html', {'form': form})


@login_required
def order_history(request):
    context = {'order_items': request.user.order.all()}
    return render(request, 'orders/history.html', context)


class OrderInvoiceDownloadView(PDFTemplateView):

    template_name = 'orders/invoice.html'

    def get_context_data(self, **kwargs):
        context = super(
            OrderInvoiceDownloadView, self).get_context_data(**kwargs)

        Order = apps.get_model('orders', 'Order')

        user = self.request.user

        order_params = {'pk': self.kwargs.get('pk')}

        if not user.is_superuser:
            order_params['user'] = user

        context.update({
            'site': Site.objects.get_current(),
            'mobile': ORDER_INVOICE_MOBILE,
            'email': ORDER_INVOICE_EMAIL,
            'order': get_object_or_404(Order, **order_params)
        })

        return context




from django.shortcuts import get_object_or_404, render

from pure_pagination import Paginator

from shop.flags.models import ProductFlag


def get_products_by_flag(request, flag_pk):

    flag = get_object_or_404(ProductFlag, pk=flag_pk)

    paginator = Paginator(flag.products.visible(), per_page=18, request=request)

    context = {
        'flag': flag,
        'products': paginator.page(request.GET.get('page', 1))
    }

    return render(request, 'products/by_flag.html', context)




from random import randint

from django.core.files import File
from django.core.urlresolvers import reverse
from django.apps import apps
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from django.utils import timezone

from sorl.thumbnail import get_thumbnail
from slugify import slugify_url
from mptt.models import MPTTModel, TreeForeignKey
from ordered_model.models import OrderedModelBase

from shop.lib import get_preview

from shop.currencies.lib import format_printable_price
from shop.currencies.models import ExchangeRate
from shop.currencies.settings import CURRENCIES, DEFAULT_CURRENCY


def get_file_upload_path(folder, filename):
    tz_now = timezone.localtime(timezone.now()).strftime('%Y_%m_%d_%H_%M')
    return '%s/%s.%s' % (folder, tz_now, filename.split('.')[-1])


def get_product_image_upload_path(instance, filename):
    return get_file_upload_path('product_images', filename)


def get_product_category_logo_upload_path(instance, filename):
    return get_file_upload_path('product_category_logos', filename)


class AbstractProductCategory(MPTTModel):

    parent = TreeForeignKey('self', null=True, blank=True,
                            related_name='children', db_index=True)

    name = models.CharField(_('Name'), max_length=255, db_index=True)

    logo = models.ImageField(
        _("Logo"), upload_to=get_product_category_logo_upload_path,
        max_length=255, blank=True, null=True)

    _slug_separator = '/'
    _full_name_separator = ' > '

    @property
    def slug(self):
        return slugify_url(self.name, separator='_')

    @property
    def ancestors(self):
        return self.get_ancestors(include_self=True)

    @property
    def full_slug(self):
        slugs = [category.slug for category in self.ancestors]
        return self._slug_separator.join(slugs)

    @property
    def full_name(self):
        try:
            names = [category.name for category in self.ancestors]
            return self._full_name_separator.join(names)
        except ValueError:
            return self.name

    full_name.fget.short_description = _('Full name')

    @property
    def preview(self):
        try:
            url = get_thumbnail(
                self.logo.file, '100x100', crop='center', quality=99).url
        except Exception:
            return '-----'

        return mark_safe('<img src="%s" style="width: 100px;" />' % url)

    preview.fget.short_description = _('Preview')

    def __unicode__(self):
        return self.full_name

    def get_absolute_url(self):
        return reverse('products:category', kwargs={
            'category_slug': self.full_slug, 'category_pk': self.pk})

    class MPTTMeta:
        order_insertion_by = ['name']

    class Meta:
        abstract = True
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')


class ProductQuerySet(models.QuerySet):

    def visible(self):
        return self.filter(is_visible=True)


class ProductManager(models.Manager):

    def get_query_set(self):
        return ProductQuerySet(self.model, using=self._db)

    def visible(self):
        return self.filter(is_visible=True)


class ProductVisibilityManager(models.Manager):

    def get_queryset(self):
        queryset = super(ProductVisibilityManager, self).get_queryset()
        return queryset.filter(is_visible=True)


class Price(object):

    def __init__(self, obj):
        self.obj = obj

    @property
    def currency(self):
        return self.obj.currency

    def printable_currency(self):
        return self.obj.get_currency_display()

    @property
    def default(self):
        return ExchangeRate.convert(
            price=self.obj.price_in_currency,
            src_currency=self.obj.currency,
            dst_currency=DEFAULT_CURRENCY)

    @property
    def printable_default(self):
        return ExchangeRate.convert(
            price=self.obj.price_in_currency,
            src_currency=self.obj.currency,
            dst_currency=DEFAULT_CURRENCY,
            printable=True)

    @property
    def initial(self):
        return self.obj.price_in_currency

    @property
    def printable_initial(self):
        return format_printable_price(self.initial, self.currency)


class AbstractProduct(models.Model):

    is_visible = models.BooleanField(_('Is visible'), default=True)

    category = models.ForeignKey(
        'products.ProductCategory', verbose_name=_("Category"),
        related_name='products', blank=False)

    title = models.CharField(_('Title'), max_length=255, blank=True)

    code = models.CharField(_('Code'), max_length=255, blank=True)

    price_in_currency = models.FloatField(_('Price'), blank=False, null=False)

    currency = models.PositiveSmallIntegerField(
        _('Currency'), choices=CURRENCIES, default=DEFAULT_CURRENCY)

    description = models.TextField(_('Description'), blank=True)

    date_created = models.DateTimeField(_("Date created"), auto_now_add=True)

    date_updated = models.DateTimeField(
        _("Date updated"), auto_now=True, db_index=True)

    logo = models.ImageField(
        _("Logo"), upload_to=get_product_image_upload_path, max_length=255,
        blank=True, null=True, editable=False)

    objects = ProductManager()
    visible = ProductVisibilityManager()

    def __init__(self, *args, **kwargs):

        super(AbstractProduct, self).__init__(*args, **kwargs)

        self.price = Price(self)

    def refresh_logo(self):

        first_image = self.images.first()

        if first_image:
            file = File(open(first_image.file.path))
            self.logo.save(first_image.file.name, file)
        else:
            self.logo.delete()

    @property
    def slug(self):
        return slugify_url(self.title, separator='_')

    @property
    def printable_code(self):
        return self.code or _('Not specified')

    @property
    def preview(self):
        return get_preview(self.logo)

    preview.fget.short_description = _('Preview')

    @classmethod
    def get_related_products(cls, category, exclude_pk, count=6):

        index = 0

        related_products = cls.objects.filter(
            is_visible=True, category=category).exclude(pk=exclude_pk)

        related_products_count = len(related_products)

        if related_products_count:

            if related_products_count > count:
                index = randint(0, related_products_count - count)

            return related_products[index:index + count]

        return []

    @property
    def related_products(self):
        if not hasattr(self, '_related_products'):
            self._related_products = self.get_related_products(
                self.category, self.pk)

        return self._related_products

    @classmethod
    def get_search_fields(cls):

        if not apps.is_installed('modeltranslation'):
            return ['code', 'title', 'description']

        fields = ['code']

        for language in dict(settings.LANGUAGES).keys():
            fields.append('title_%s' % language)
            fields.append('description_%s' % language)

        return fields

    def __unicode__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('products:info', kwargs={
            'product_slug': self.slug, 'product_pk': self.id})

    class Meta:
        abstract = True
        ordering = ['-id']
        verbose_name = _('Product')
        verbose_name_plural = _('Products')


class AbstractProductImage(OrderedModelBase):

    product = models.ForeignKey(
        'products.Product', related_name='images', verbose_name=_("Product"))

    file = models.ImageField(
        _("File"), upload_to=get_product_image_upload_path, max_length=255)

    order = models.PositiveIntegerField(_('Ordering'), default=0)

    order_field_name = 'order'
    order_with_respect_to = 'product'

    def __unicode__(self):
        return self.product.title

    @property
    def preview(self):
        return get_preview(self.file)

    preview.fget.short_description = _('Preview')

    class Meta:
        abstract = True
        ordering = ('order', 'id', )
        verbose_name = _('Product image')
        verbose_name_plural = _('Product images')




DELIVERY_METHOD_SELF = 1
DELIVERY_METHOD_NOVA_POSHTA = 2
DELIVERY_METHOD_INTIME = 3
DELIVERY_METHOD_GUNSEL = 4
DELIVERY_METHOD_AVTOLUKS = 5
DELIVERY_METHOD_OTHER = 6
DELIVERY_METHOD_MEEST_EXPRESS = 7


PAYMENT_METHOD_CASH = 1
PAYMENT_METHOD_CASHLESS = 2
PAYMENT_METHOD_COD = 3
PAYMENT_METHOD_PRIVAT24 = 4


STATUS_NOT_REVIEWED = 0
STATUS_PROCESSING = 1
STATUS_CANCELED = 2
STATUS_COMPLETED = 3




from django.conf import settings

from shop.currencies.constants import CURRENCY_UAH, DEFAULT_CURRENCIES


CURRENCIES = getattr(settings, 'CURRENCIES', DEFAULT_CURRENCIES)

DEFAULT_CURRENCY = getattr(settings, 'DEFAULT_CURRENCY', CURRENCY_UAH)

CURRENCY_SESSION_KEY = getattr(settings, 'CURRENCIES', 'CURRENCY')




from django.conf.urls import url
from django.contrib.auth.decorators import login_required

from shop.orders import views


app_name = 'orders'


urlpatterns = [

    url(r'^checkout/$', views.checkout, name='checkout'),

    url(r'^order-history/$', views.order_history, name='history'),

    url(r'^invoice/(?P<pk>\d+)/download/$',
        login_required(views.OrderInvoiceDownloadView.as_view()),
        name='download-invoice'),

]




from django.apps import apps
from django.shortcuts import render, get_object_or_404
from django.http.response import HttpResponse
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.http import require_POST, require_GET

from shop.offers.forms import ProductPriceOfferForm
from shop.offers.lib import send_new_price_offer_notification


@require_GET
def get_price_offer_modal(request, product_pk):

    user = request.user

    Product = apps.get_model('products', 'Product')

    product = get_object_or_404(Product, pk=product_pk)

    if user.is_authenticated():
        profile = getattr(user, 'profile', None)
        initial = {
            'name': user.get_full_name(),
            'email': user.email,
            'mobile': profile.mobile if profile is not None else ''
        }
    else:
        initial = {}

    form = ProductPriceOfferForm(initial=initial)

    context = {
        'form': form,
        'product': product
    }

    return render(request, 'offers/price_offer_modal.html', context)


@require_POST
def send_price_offer(request, product_pk):

    Product = apps.get_model('products', 'Product')

    product = get_object_or_404(Product, pk=product_pk)

    form = ProductPriceOfferForm(request.POST)

    if form.is_valid():

        offer = form.save(commit=False)

        if request.user.is_authenticated():
            offer.user = request.user

        offer.product = product
        offer.save()

        send_new_price_offer_notification(offer)

        return HttpResponse(_('Offer successfully added'))

    context = {
        'form': form,
        'product': product
    }

    return render(request, 'offers/price_offer_form.html', context, status=400)




from django import forms

from natsort import natsorted

from shop.attributes.models import ProductAttributeValue
from shop.attributes.constants import ATTR_TYPE_SELECT


class SearchProductAttrMixin(object):

    def __init__(self, products, category, *args, **kwargs):

        self._attributes = category.attributes.all().filter(
                type=ATTR_TYPE_SELECT, is_filter=True).order_by('name')

        super(SearchProductAttrMixin, self).__init__(
            products, category, *args, **kwargs)

        options = self._get_available_options()

        for attr in self._attributes:
            self.fields[attr.full_slug] = self._build_attr_field(attr, options)

    def _filter(self, products):

        products = super(SearchProductAttrMixin, self)._filter(products)

        if not self.data:
            return products

        for attr in self._attributes:

            values = self.data.getlist(attr.full_slug)

            if not values:
                continue

            products = products.filter(
                attribute_values__value_option__in=values)

        return products

    def _build_attr_field(self, attr, available_options):

        options = natsorted(available_options[attr.pk], key=lambda o: o.option)

        choices = [(o.id, o.option) for o in options]

        return forms.MultipleChoiceField(
            widget=forms.CheckboxSelectMultiple, label=attr.name,
            required=False, choices=choices)

    def _get_available_options(self):

        added_options = []

        options = {attr.pk: [] for attr in self._attributes}

        attr_values = ProductAttributeValue.objects.filter(
            attribute__in=self._attributes, product__in=self._products
        ).select_related('value_option')

        for value in attr_values:

            option = value.value_option

            if option not in added_options:
                added_options.append(option)
                options[value.attribute_id].append(option)

        return options



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-03-22 14:46
from __future__ import unicode_literals

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('products', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('payment_method', models.PositiveIntegerField(choices=[(1, 'Cash payment'), (2, 'Cashless payment'), (3, 'C.O.D.'), (4, 'Privat24 Payment')], default=None, verbose_name='Payment method')),
                ('delivery_method', models.PositiveIntegerField(choices=[(1, 'Self delivery'), (2, 'Nova Poshta'), (7, 'Meest Express'), (6, 'Other')], default=None, verbose_name='Delivery method')),
                ('status', models.PositiveIntegerField(choices=[(0, 'Not reviewed'), (1, 'Processing'), (2, 'Canceled'), (3, 'Completed')], default=0, verbose_name='Status')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('surname', models.CharField(max_length=255, verbose_name='Surname')),
                ('post_office', models.CharField(blank=True, max_length=255, null=True, verbose_name='Address and number of post office')),
                ('mobile', models.CharField(max_length=255, verbose_name='Mobile phone')),
                ('email', models.EmailField(max_length=255, verbose_name='Email')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('comment', models.TextField(blank=True, default=b'', max_length=1000, verbose_name='Comment')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='order', to=settings.AUTH_USER_MODEL, verbose_name='Customer')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Order',
                'verbose_name_plural': 'Orders',
            },
        ),
        migrations.CreateModel(
            name='OrderProduct',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=255, verbose_name='Product title')),
                ('price', models.FloatField(verbose_name='Price (uah)')),
                ('qty', models.PositiveIntegerField(default=1, validators=[django.core.validators.MinValueValidator(1)], verbose_name='Quantity')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='products', to='orders.Order', verbose_name='Order')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='products.Product', verbose_name='Product')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Order product',
                'verbose_name_plural': 'Order products',
            },
        ),
    ]




from django.apps import apps
from django.conf import settings
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from ckeditor.widgets import CKEditorWidget
from mptt.admin import MPTTModelAdmin
from modeltranslation.admin import TranslationAdmin

from shop.products.settings import (
    IS_CKEDITOR_ENABLED_FOR_PRODUCT_DESCRIPTION)

from shop.currencies.settings import CURRENCIES
from shop.currencies.models import ExchangeRate

from shop.products.admin.forms import ProductForm, ProductImageInline
from shop.products.admin import views
from shop.products.admin import actions

from shop.lib import get_show_on_site_link


class ProductAdmin(TranslationAdmin):

    inlines = [ProductImageInline]

    actions = [actions.refresh_products_logos]

    form = ProductForm

    list_display = [
        'id', 'title', 'category', 'printable_price', 'code', 'date_updated',
        'get_show_link', 'preview'
    ]

    list_display_links = ('title', )

    list_filter = ('category', )

    ordering = ['-id']

    search_fields = apps.get_model('products', 'Product').get_search_fields()

    def __init__(self, *args, **kwargs):

        if IS_CKEDITOR_ENABLED_FOR_PRODUCT_DESCRIPTION:
            self.formfield_overrides = {
                models.TextField: {'widget': CKEditorWidget}
            }

        super(ProductAdmin, self).__init__(*args, **kwargs)

    def get_show_link(self, item):
        return get_show_on_site_link(item.get_absolute_url())

    get_show_link.short_description = _('Show on site')

    def printable_price(self, item):

        html = '<b>%s</b><br />' % item.price.printable_initial

        for currency_id, currency_name in CURRENCIES:
            if currency_id is item.currency:
                continue

            price = str(ExchangeRate.convert(
                item.price.initial, item.currency, currency_id,
                format_price=True))

            html += '<small>%s %s</small> <br />' % (price, currency_name)

        return mark_safe(html)

    printable_price.short_description = _('Price')

    def save_model(self, request, obj, form, change):

        self._product = product = form.save()

        images = form.cleaned_data.get('images', [])

        for image in images:
            if image:
                product.images.create(file=image)

    def save_related(self, request, form, formsets, change):
        super(ProductAdmin, self).save_related(request, form, formsets, change)
        self._product.refresh_logo()


class ProductCategoryAdmin(MPTTModelAdmin, TranslationAdmin):

    list_display = ('full_name', 'product_count', 'preview', )

    def product_count(self, item):
        return item.products.count()

    product_count.short_description = _('Product count')


if 'shop.products' in settings.INSTALLED_APPS:
    from shop.products.models import Product, ProductCategory

    admin.site.register(Product, ProductAdmin)
    admin.site.register(ProductCategory, ProductCategoryAdmin)




from django import template


register = template.Library()


@register.assignment_tag(takes_context=True)
def get_comparison(context, session_key='comparison'):
    from shop.comparison import Comparison
    return Comparison(context['request'].session, session_key=session_key)




from django.utils.translation import ugettext_lazy as _


def refresh_products_logos(modeladmin, request, queryset):
    for product in queryset:
        product.refresh_logo()

refresh_products_logos.short_description = _('Refresh product logos')




from django.apps import apps
from django.shortcuts import get_object_or_404, redirect, render
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.decorators import login_required

from shop.wishlist.models import WishListItem


@login_required
def index(request):
    context = {'wishlist_items': request.user.wishlist_items.all()}
    return render(request, 'wishlist/index.html', context)


@login_required
def add(request):

    Product = apps.get_model('products', 'Product')

    product = get_object_or_404(
        Product, pk=request.GET.get('product_pk'))

    if WishListItem.objects.filter(product=product).exists():
        message = _('%s already in your wish list')
        messages.success(request, message % product.title)

    else:
        WishListItem.objects.create(
            user=request.user, product=product, product_title=product.title,
            qty=request.GET.get('qty', 1))

        message = _('%s addet to wish list')
        messages.success(request, message % product.title)

    return redirect(request.GET.get('next', 'home'))


@login_required
def remove(request):

    item = get_object_or_404(WishListItem, pk=request.GET.get('pk'))
    item.delete()

    message = _('%s was removed from wish list')
    messages.success(request, message % item.product_title)

    return redirect(request.GET.get('next', 'home'))




from django.db import models
from django.core.cache import cache
from django.utils.translation import ugettext_lazy as _

from shop.lib import format_number
from shop.currencies.lib import format_printable_price
from shop.currencies.constants import CURRENCY_UAH, CURRENCY_EUR, CURRENCY_USD
from shop.currencies.settings import CURRENCIES


class ExchangeRate(models.Model):

    currency = models.PositiveIntegerField(
        _('Currency'), choices=CURRENCIES, unique=True)

    value = models.FloatField(_('Value'), default=0)

    def __unicode__(self):
        return '%s: %s' % (self.get_currency_display(), self.value)

    @classmethod
    def get_exchange_rates(cls):

        exchange_rates = cache.get('exchange_rates', {})

        if not exchange_rates:

            exchange_rates = {c: 1 for c, n in CURRENCIES}

            for exchange_rate in cls.objects.all():
                exchange_rates[exchange_rate.currency] = exchange_rate.value

            cache.set('exchange_rates', exchange_rates, 240)

        return exchange_rates

    @classmethod
    def convert(cls, price, src_currency, dst_currency,
                format_price=False, printable=False):

        exchange_rates = cls.get_exchange_rates()

        if src_currency is dst_currency:
            new_price = price

        elif src_currency is CURRENCY_UAH and dst_currency is CURRENCY_USD:
            new_price = price / exchange_rates[CURRENCY_USD]

        elif src_currency is CURRENCY_UAH and dst_currency is CURRENCY_EUR:
            new_price = price / exchange_rates[CURRENCY_EUR]

        elif src_currency is CURRENCY_USD and dst_currency is CURRENCY_UAH:
            new_price = price * exchange_rates[CURRENCY_USD]

        elif src_currency is CURRENCY_EUR and dst_currency is CURRENCY_UAH:
            new_price = price * exchange_rates[CURRENCY_EUR]

        elif src_currency is CURRENCY_USD and dst_currency is CURRENCY_EUR:
            new_price = (price / exchange_rates[CURRENCY_EUR]) * \
                   exchange_rates[CURRENCY_USD]

        elif src_currency is CURRENCY_EUR and dst_currency is CURRENCY_USD:
            new_price = (price / exchange_rates[CURRENCY_USD]) * \
                   exchange_rates[CURRENCY_EUR]
        else:
            raise ValueError(
                'Unknown currency %s/%s' % (dst_currency, src_currency))

        if format_price and not printable:
            new_price = format_number(new_price)

        if printable:
            new_price = format_printable_price(new_price, dst_currency)

        return new_price

    class Meta:
        verbose_name = _('Exchange rate')
        verbose_name_plural = _('Exchange rate')




from django.apps import apps
from django import forms
from django.utils.translation import ugettext_lazy as _

from model_search import model_search


class SearchProductForm(forms.Form):

    ORDER_BY_PRICE_HIGH_TO_LOW = 'price'
    ORDER_BY_PRICE_LOW_TO_HIGH = '-price'
    ORDER_BY_NEWEST = 'newest'
    ORDER_BY_A_TO_Z = 'title'
    ORDER_BY_Z_TO_A = '-title'

    ORDER_CHOICES = (
        (ORDER_BY_NEWEST, _('Newest')),
        (ORDER_BY_PRICE_HIGH_TO_LOW, _('Price (high-low)')),
        (ORDER_BY_PRICE_LOW_TO_HIGH, _('Price (low-high)')),
        (ORDER_BY_A_TO_Z, _('A-Z')),
        (ORDER_BY_Z_TO_A, _('Z-A')),
    )

    query = forms.CharField(label=_('Query'), required=False)

    order_by = forms.ChoiceField(
        label=_('Order by'), required=False, choices=ORDER_CHOICES,
        initial=ORDER_BY_NEWEST)

    def __init__(self, products, category=None, *args, **kwargs):
        super(SearchProductForm, self).__init__(*args, **kwargs)

        self._category = category

        if self.is_valid():
            products = self._filter(products)
            products = self._order(products)
        else:
            products = []

        self._products = products

    def _filter(self, products):

        if self._category is not None:
            categories = self._category.get_descendants(include_self=True)
            products = products.filter(category__in=categories)

        query = self.cleaned_data.get('query')

        if query:
            Product = apps.get_model('products', 'Product')
            products = model_search(query, products, Product.get_search_fields())

        return products

    def _order(self, products):

        order_by = self.cleaned_data.get('order_by')

        if not order_by or order_by == self.ORDER_BY_NEWEST:
            return products.order_by('-id')

        if order_by == self.ORDER_BY_A_TO_Z:
            return products.order_by('title')

        if order_by == self.ORDER_BY_Z_TO_A:
            return products.order_by('-title')

        if order_by == self.ORDER_BY_PRICE_HIGH_TO_LOW:
            return sorted(products, key=lambda p: p.price.default,
                          reverse=True)

        if order_by == self.ORDER_BY_PRICE_LOW_TO_HIGH:
            return sorted(products, key=lambda p: p.price.default)

    def get_objects(self):
        return self._products




from django.apps import apps
from django.shortcuts import render, get_object_or_404, redirect
from django.contrib import messages
from django.utils.translation import ugettext as _

from shop.comparison import Comparison


MAX_NUMBER_OF_COMPARISON_PRODUCTS = 4


def index(request):

    comparison = Comparison(request.session)

    category = _get_category_from_request(request)

    products = comparison.get_products(category)

    context = {
        'category': category,
        'products': products,
        'attributes': _format_attributes(category, products)
    }
    return render(request, 'comparison/index.html', context)


def add(request):

    product = _get_product_from_request(request)

    comparison = Comparison(request.session)

    max_number = MAX_NUMBER_OF_COMPARISON_PRODUCTS

    if len(comparison.get_products(product.category)) == max_number:
        message = _('Only %s products can be compared')
        messages.error(request, message % max_number)

    elif comparison.get_product(product.pk):
        message = _('%s already in comparison list')
        messages.error(request, message % product.title)

    else:
        comparison.add(product)

        message = _('%s added to comparison')
        messages.success(request, message % product.title)

    return redirect(request.GET.get('next', 'home'))


def remove(request):

    product = _get_product_from_request(request)

    comparison = Comparison(request.session)
    comparison.remove(product)

    message = _('%s removed from comparison')
    messages.success(request, message % product.title)

    return redirect(request.GET.get('next', 'home'))


def _format_attributes(category, products):
    attrs = []

    for attr in category.attributes.all():
        attrs.append({
            'name': attr.name,
            'values': [getattr(p.attr, attr.slug) for p in products]
        })

    return attrs


def _get_product_from_request(request):
    Product = apps.get_model('products', 'Product')
    return get_object_or_404(Product, pk=request.GET.get('product_pk'))


def _get_category_from_request(request):
    ProductCategory = apps.get_model('products', 'ProductCategory')
    return get_object_or_404(
        ProductCategory, pk=request.GET.get('category_pk'))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-03-22 14:45
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('products', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ProductPriceOffer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.PositiveIntegerField(choices=[(0, 'Not reviewed'), (1, 'Processing'), (2, 'Canceled'), (3, 'Completed')], default=0, verbose_name='Status')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('mobile', models.CharField(max_length=255, verbose_name='Mobile phone')),
                ('email', models.EmailField(max_length=255, verbose_name='Email')),
                ('text', models.TextField(max_length=1000, verbose_name='Offer')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='price_offers', to='products.Product', verbose_name='Product')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='offers', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'ordering': ['-date_created'],
                'verbose_name': 'Product price offer',
                'verbose_name_plural': 'Product price offers',
            },
        ),
    ]




from modeltranslation.translator import translator, TranslationOptions

from shop.flags.models import ProductFlag


class ProductFlagTranslationOptions(TranslationOptions):

    fields = ('title', )


translator.register(ProductFlag, ProductFlagTranslationOptions)




from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _


class ProductPriceOffer(models.Model):

    STATUS_NOT_REVIEWED = 0
    STATUS_PROCESSING = 1
    STATUS_CANCELED = 2
    STATUS_COMPLETED = 3

    STATUS_CHOICES = (
        (STATUS_NOT_REVIEWED, _('Not reviewed')),
        (STATUS_PROCESSING, _('Processing')),
        (STATUS_CANCELED, _('Canceled')),
        (STATUS_COMPLETED, _('Completed')),
    )

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='offers', null=True, blank=True,
        verbose_name=_('User'))

    status = models.PositiveIntegerField(
        _('Status'), null=False, choices=STATUS_CHOICES,
        default=STATUS_NOT_REVIEWED)

    product = models.ForeignKey(
        'products.Product', related_name='price_offers',
        verbose_name=_("Product"))

    name = models.CharField(
        _("Name"), max_length=255, blank=False, null=False)

    mobile = models.CharField(
        _("Mobile phone"), max_length=255, blank=False, null=False)

    email = models.EmailField(
        _("Email"), max_length=255, blank=False, null=False)

    text = models.TextField(_('Offer'), blank=False, max_length=1000)

    date_created = models.DateTimeField(_("Date created"), auto_now_add=True)

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ['-date_created']
        verbose_name = _('Product price offer')
        verbose_name_plural = _('Product price offers')




import re

from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from sorl.thumbnail import get_thumbnail


def round_number(value, decimal_places=2, down=False):

    assert decimal_places > 0
    factor = 1.0 ** decimal_places
    sign = -1 if value < 0 else 1
    return int(value * factor + sign * (0 if down else 0.5)) / factor


def format_number(value):

    append_comma = lambda match_object: "%s," % match_object.group(0)

    value = "%.2f" % float(value)
    value = re.sub("(\d)(?=(\d{3})+\.)", append_comma, value)

    return value


def format_price(price, round_price=False):
    price = float(price)
    return format_number(round_number(price) if round_price else price)


def get_show_on_site_link(url):
    return mark_safe(
        '<a href="%s" target="_blank">%s</a>' % (url, _('Show on site')))


def get_preview(field, width=100):

    if not field:
        return '-----'
    try:
        url = get_thumbnail(field.file, str(width)).url
    except IOError:
        return _('Image not found')

    return mark_safe('<img src="%s" style="width: %spx;" title="%s" />' % (
        url, width, field.path))
